@@ -162,9 +162,9 @@ class LinkerEntry:
             linker_writer._write_symbol(path_cname, ".")
 
     def emit_path(self, linker_writer: "LinkerWriter"):
-        assert (
-            self.object_path is not None
-        ), f"{self.segment.name}, {self.segment.rom_start}"
+        assert self.object_path is not None, (
+            f"{self.segment.name}, {self.segment.rom_start}"
+        )
 
         if self.noload and self.bss_contains_common:
             linker_writer._write_object_path_section(
@@ -204,8 +204,10 @@ class LinkerWriter:
         if not self.is_partial:
             self._writeln(f"__romPos = {options.opts.ld_rom_start};")
 
-            if options.opts.gp is not None:
-                self._writeln("_gp = " + f"0x{options.opts.gp:X};")
+            if options.opts.ld_gp_expression is not None:
+                self._writeln(f"_gp = {options.opts.ld_gp_expression};")
+            elif options.opts.gp is not None:
+                self._writeln(f"_gp = 0x{options.opts.gp:X};")
 
     # Write a series of statements which compute a symbol that represents the highest address among a list of segments' end addresses
     def write_max_vram_end_sym(self, symbol: str, overlays: List[Segment]):
@@ -235,9 +237,9 @@ class LinkerWriter:
             return
 
         section_entries: OrderedDict[str, List[LinkerEntry]] = OrderedDict()
-        for l in segment.section_order:
-            if l in options.opts.section_order:
-                section_entries[l] = []
+        for section_name in segment.section_order:
+            if section_name in options.opts.section_order:
+                section_entries[section_name] = []
 
         # Add all entries to section_entries
         prev_entry = None
@@ -299,7 +301,7 @@ class LinkerWriter:
 
         # To keep track which sections has been started
         started_sections: Dict[str, bool] = {
-            l: False for l in options.opts.section_order
+            section_name: False for section_name in options.opts.section_order
         }
 
         # Find where sections are last seen
@@ -387,14 +389,19 @@ class LinkerWriter:
 
             self._begin_segment(segment, seg_name, noload=False, is_first=is_first)
 
-            for l in segment.section_order:
-                if l not in options.opts.section_order:
+            for section_name in segment.section_order:
+                if section_name not in options.opts.section_order:
                     continue
-                if l == ".bss":
+                if section_name == ".bss":
                     continue
 
                 entry = LinkerEntry(
-                    segment, [], segments_path / f"{seg_name}.o", l, l, noload=False
+                    segment,
+                    [],
+                    segments_path / f"{seg_name}.o",
+                    section_name,
+                    section_name,
+                    noload=False,
                 )
                 self.dependencies_entries.append(entry)
                 entry.emit_entry(self)
@@ -437,9 +444,9 @@ class LinkerWriter:
         seg_name = segment.get_cname()
 
         section_entries: OrderedDict[str, List[LinkerEntry]] = OrderedDict()
-        for l in segment.section_order:
-            if l in options.opts.section_order:
-                section_entries[l] = []
+        for section_name in segment.section_order:
+            if section_name in options.opts.section_order:
+                section_entries[section_name] = []
 
         # Add all entries to section_entries
         prev_entry = None
@@ -592,7 +599,7 @@ class LinkerWriter:
         if not noload:
             seg_rom_start = get_segment_rom_start(seg_name)
             line += f" AT({seg_rom_start})"
-        if options.opts.emit_subalign and segment.subalign != None:
+        if options.opts.emit_subalign and segment.subalign is not None:
             line += f" SUBALIGN({segment.subalign})"
 
         self._writeln(line)
@@ -634,7 +641,7 @@ class LinkerWriter:
         if noload:
             line += " (NOLOAD)"
         line += " :"
-        if options.opts.emit_subalign and segment.subalign != None:
+        if options.opts.emit_subalign and segment.subalign is not None:
             line += f" SUBALIGN({segment.subalign})"
 
         self._writeln(line)
